@@ -3,14 +3,15 @@
 # Use of this source code is governed by a MIT License
 # license that can be found in the LICENSE file.
 
-#Tests M3U8 class to make sure all attributes and methods use the correct
-#data returned from parser.parse()
+# Tests M3U8 class to make sure all attributes and methods use the correct
+# data returned from parser.parse()
 
 import arrow
 import datetime
 import m3u8
 import playlists
-from m3u8.model import Segment
+from m3u8.model import Segment, Key
+
 
 def test_target_duration_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
@@ -18,23 +19,28 @@ def test_target_duration_attribute():
 
     assert '1234567' == obj.target_duration
 
+
 def test_media_sequence_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
     mock_parser_data(obj, {'media_sequence': '1234567'})
 
     assert '1234567' == obj.media_sequence
 
+
 def test_program_date_time_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST_WITH_PROGRAM_DATE_TIME)
 
     assert arrow.get('2014-08-13T13:36:33+00:00').datetime == obj.program_date_time
 
+
 def test_program_date_time_attribute_for_each_segment():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST_WITH_PROGRAM_DATE_TIME)
 
     first_program_date_time = arrow.get('2014-08-13T13:36:33+00:00').datetime
     for idx, segment in enumerate(obj.segments):
-        assert segment.program_date_time == first_program_date_time + datetime.timedelta(seconds=idx * 3)
+        assert segment.program_date_time == first_program_date_time + \
+            datetime.timedelta(seconds=idx * 3)
+
 
 def test_program_date_time_attribute_with_discontinuity():
     obj = m3u8.M3U8(playlists.DISCONTINUITY_PLAYLIST_WITH_PROGRAM_DATE_TIME)
@@ -57,6 +63,7 @@ def test_segment_discontinuity_attribute():
     assert segments[5].discontinuity == True
     assert segments[6].discontinuity == False
 
+
 def test_segment_cue_out_attribute():
     obj = m3u8.M3U8(playlists.CUE_OUT_PLAYLIST)
     segments = obj.segments
@@ -65,39 +72,65 @@ def test_segment_cue_out_attribute():
     assert segments[2].cue_out == True
     assert segments[3].cue_out == False
 
-def test_segment_scte35_attribute():
-    obj = m3u8.M3U8(playlists.CUE_OUT_WITH_SCTE35_PLAYLIST)
+
+def test_segment_elemental_scte35_attribute():
+    obj = m3u8.M3U8(playlists.CUE_OUT_ELEMENTAL_PLAYLIST)
     segments = obj.segments
     assert segments[4].cue_out == True
     assert segments[9].cue_out == False
-    assert segments[4].scte35 == '/DAlAAAAAAAAAP/wFAUAAAABf+//wpiQkv4ARKogAAEBAQAAQ6sodg==' 
+    assert segments[4].scte35 == '/DAlAAAAAAAAAP/wFAUAAAABf+//wpiQkv4ARKogAAEBAQAAQ6sodg=='
+
+def test_segment_envivio_scte35_attribute():
+    obj = m3u8.M3U8(playlists.CUE_OUT_ENVIVIO_PLAYLIST)
+    segments = obj.segments
+    assert segments[3].cue_out == True
+    assert segments[4].scte35 == '/DAlAAAENOOQAP/wFAUBAABrf+//N25XDf4B9p/gAAEBAQAAxKni9A=='
+    assert segments[5].scte35 == '/DAlAAAENOOQAP/wFAUBAABrf+//N25XDf4B9p/gAAEBAQAAxKni9A=='
+    assert segments[7].cue_out == False
+
+def test_keys_on_clear_playlist():
+    obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
+
+    assert len(obj.keys) == 1
+    assert obj.keys[0] == None
+
+
+def test_keys_on_simple_encrypted_playlist():
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS)
+
+    assert len(obj.keys) == 1
+    assert obj.keys[0].uri == "https://priv.example.com/key.php?r=52"
+
 
 def test_key_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
-    data = {'key': {'method': 'AES-128',
+    data = {'keys': [{'method': 'AES-128',
                     'uri': '/key',
-                    'iv': 'foobar'}}
+                    'iv': 'foobar'}]}
     mock_parser_data(obj, data)
 
-    assert 'Key' == obj.key.__class__.__name__
-    assert 'AES-128' == obj.key.method
-    assert '/key' == obj.key.uri
-    assert 'foobar' == obj.key.iv
+    assert 'Key' == obj.keys[0].__class__.__name__
+    assert 'AES-128' == obj.keys[0].method
+    assert '/key' == obj.keys[0].uri
+    assert 'foobar' == obj.keys[0].iv
+
 
 def test_key_attribute_on_none():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
     mock_parser_data(obj, {})
 
-    assert None == obj.key
+    assert len(obj.keys) == 0
+
 
 def test_key_attribute_without_initialization_vector():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
-    mock_parser_data(obj, {'key': {'method': 'AES-128',
-                                   'uri': '/key'}})
+    mock_parser_data(obj, {'keys': [{'method': 'AES-128',
+                                   'uri': '/key'}]})
+
+    assert 'AES-128' == obj.keys[0].method
+    assert '/key' == obj.keys[0].uri
+    assert None == obj.keys[0].iv
 
-    assert 'AES-128' == obj.key.method
-    assert '/key' == obj.key.uri
-    assert None == obj.key.iv
 
 def test_segments_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
@@ -114,6 +147,7 @@ def test_segments_attribute():
     assert 'First Segment' == obj.segments[0].title
     assert 1500 == obj.segments[0].duration
 
+
 def test_segments_attribute_without_title():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
     mock_parser_data(obj, {'segments': [{'uri': '/foo/bar-1.ts',
@@ -125,6 +159,7 @@ def test_segments_attribute_without_title():
     assert 1500 == obj.segments[0].duration
     assert None == obj.segments[0].title
 
+
 def test_segments_attribute_without_duration():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
     mock_parser_data(obj, {'segments': [{'uri': '/foo/bar-1.ts',
@@ -136,6 +171,7 @@ def test_segments_attribute_without_duration():
     assert 'Segment title' == obj.segments[0].title
     assert None == obj.segments[0].duration
 
+
 def test_segments_attribute_with_byterange():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
     mock_parser_data(obj, {'segments': [{'uri': '/foo/bar-1.ts',
@@ -150,6 +186,7 @@ def test_segments_attribute_with_byterange():
     assert 1500 == obj.segments[0].duration
     assert '76242@0' == obj.segments[0].byterange
 
+
 def test_segment_attribute_with_multiple_keys():
     obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS)
 
@@ -168,6 +205,7 @@ def test_is_variant_attribute():
     mock_parser_data(obj, {'is_variant': True})
     assert obj.is_variant
 
+
 def test_is_endlist_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
     mock_parser_data(obj, {'is_endlist': False})
@@ -177,6 +215,7 @@ def test_is_endlist_attribute():
     mock_parser_data(obj, {'is_endlist': True})
     assert obj.is_endlist
 
+
 def test_is_i_frames_only_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
     mock_parser_data(obj, {'is_i_frames_only': False})
@@ -185,6 +224,7 @@ def test_is_i_frames_only_attribute():
     mock_parser_data(obj, {'is_i_frames_only': True})
     assert obj.is_i_frames_only
 
+
 def test_playlists_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
     data = {'playlists': [{'uri': '/url/1.m3u8',
@@ -200,8 +240,8 @@ def test_playlists_attribute():
             'media': [{'type': 'VIDEO', 'name': 'High', 'group_id': 'high'},
                       {'type': 'VIDEO', 'name': 'Low', 'group_id': 'low',
                        'default': 'YES', 'autoselect': 'YES'}
-                     ]
-           }
+                      ]
+            }
     mock_parser_data(obj, data)
 
     assert 2 == len(obj.playlists)
@@ -238,6 +278,7 @@ def test_playlists_attribute():
 
     assert [] == obj.iframe_playlists
 
+
 def test_playlists_attribute_without_program_id():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
     mock_parser_data(obj, {'playlists': [{'uri': '/url/1.m3u8',
@@ -251,6 +292,7 @@ def test_playlists_attribute_without_program_id():
     assert None == obj.playlists[0].stream_info.codecs
     assert None == obj.playlists[0].stream_info.program_id
 
+
 def test_playlists_attribute_with_resolution():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST_WITH_RESOLUTION)
 
@@ -258,6 +300,7 @@ def test_playlists_attribute_with_resolution():
     assert (512, 288) == obj.playlists[0].stream_info.resolution
     assert None == obj.playlists[1].stream_info.resolution
 
+
 def test_iframe_playlists_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
     data = {
@@ -286,6 +329,7 @@ def test_iframe_playlists_attribute():
     assert None == obj.iframe_playlists[1].iframe_stream_info.resolution
     assert 'avc1.4d400d' == obj.iframe_playlists[1].iframe_stream_info.codecs
 
+
 def test_version_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
     mock_parser_data(obj, {'version': '2'})
@@ -294,6 +338,7 @@ def test_version_attribute():
     mock_parser_data(obj, {})
     assert None == obj.version
 
+
 def test_allow_cache_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
     mock_parser_data(obj, {'allow_cache': 'no'})
@@ -302,6 +347,7 @@ def test_allow_cache_attribute():
     mock_parser_data(obj, {})
     assert None == obj.allow_cache
 
+
 def test_files_attribute_should_list_all_files_including_segments_and_key():
     obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS)
     files = [
@@ -309,39 +355,35 @@ def test_files_attribute_should_list_all_files_including_segments_and_key():
         'http://media.example.com/fileSequence52-1.ts',
         'http://media.example.com/fileSequence52-2.ts',
         'http://media.example.com/fileSequence52-3.ts',
-        ]
+    ]
     assert files == obj.files
 
+
 def test_vod_playlist_type_should_be_imported_as_a_simple_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST_WITH_VOD_PLAYLIST_TYPE)
     assert obj.playlist_type == 'vod'
 
+
 def test_event_playlist_type_should_be_imported_as_a_simple_attribute():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST_WITH_EVENT_PLAYLIST_TYPE)
     assert obj.playlist_type == 'event'
 
+
 def test_independent_segments_should_be_true():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST_WITH_INDEPENDENT_SEGMENTS)
     assert obj.is_independent_segments
 
+
 def test_independent_segments_should_be_false():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST_WITH_EVENT_PLAYLIST_TYPE)
     assert not obj.is_independent_segments
 
+
 def test_no_playlist_type_leaves_attribute_empty():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
     assert obj.playlist_type is None
 
 
-# dump m3u8
-
-def test_dumps_should_build_same_string():
-    playlists_model = [playlists.PLAYLIST_WITH_NON_INTEGER_DURATION, playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV]
-    for playlist in playlists_model:
-        obj = m3u8.M3U8(playlist)
-        expected = playlist.replace(', IV', ',IV').strip()
-        assert expected == obj.dumps().strip()
-
 def test_dump_playlists_with_resolution():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST_WITH_RESOLUTION)
 
@@ -349,26 +391,29 @@ def test_dump_playlists_with_resolution():
 
     assert expected == obj.dumps().strip().splitlines()
 
+
 def test_dump_should_build_file_with_same_content(tmpdir):
     obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV)
 
-    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV.replace(', IV', ',IV').strip()
+    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_SORTED.replace(', IV', ',IV').strip()
     filename = str(tmpdir.join('playlist.m3u8'))
 
     obj.dump(filename)
 
     assert_file_content(filename, expected)
 
+
 def test_dump_should_create_sub_directories(tmpdir):
     obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV)
 
-    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV.replace(', IV', ',IV').strip()
+    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_SORTED.replace(', IV', ',IV').strip()
     filename = str(tmpdir.join('subdir1', 'subdir2', 'playlist.m3u8'))
 
     obj.dump(filename)
 
     assert_file_content(filename, expected)
 
+
 def test_dump_should_work_for_variant_streams():
     obj = m3u8.M3U8(playlists.VARIANT_PLAYLIST)
 
@@ -376,6 +421,7 @@ def test_dump_should_work_for_variant_streams():
 
     assert expected == obj.dumps().strip()
 
+
 def test_dump_should_work_for_variant_playlists_with_iframe_playlists():
     obj = m3u8.M3U8(playlists.VARIANT_PLAYLIST_WITH_IFRAME_PLAYLISTS)
 
@@ -383,6 +429,7 @@ def test_dump_should_work_for_variant_playlists_with_iframe_playlists():
 
     assert expected == obj.dumps().strip()
 
+
 def test_dump_should_work_for_iframe_playlists():
     obj = m3u8.M3U8(playlists.IFRAME_PLAYLIST)
 
@@ -406,11 +453,13 @@ def test_dump_should_work_for_iframe_playlists():
     # hence IFRAME_PLAYLIST dump from IFRAME_PLAYLIST2 parse.
     assert expected == obj.dumps().strip()
 
+
 def test_dump_should_include_program_date_time():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST_WITH_PROGRAM_DATE_TIME)
 
     assert "EXT-X-PROGRAM-DATE-TIME:2014-08-13T13:36:33+00:00" in obj.dumps().strip()
 
+
 def test_dump_should_work_for_playlists_using_byteranges():
     obj = m3u8.M3U8(playlists.PLAYLIST_USING_BYTERANGES)
 
@@ -418,39 +467,98 @@ def test_dump_should_work_for_playlists_using_byteranges():
 
     assert expected == obj.dumps().strip()
 
+
 def test_should_dump_with_endlist_tag():
     obj = m3u8.M3U8(playlists.SLIDING_WINDOW_PLAYLIST)
-    obj.is_endlist= True
+    obj.is_endlist = True
 
     assert '#EXT-X-ENDLIST' in obj.dumps().splitlines()
 
+
 def test_should_dump_without_endlist_tag():
     obj = m3u8.M3U8(playlists.SIMPLE_PLAYLIST)
     obj.is_endlist = False
 
-    expected  = playlists.SIMPLE_PLAYLIST.strip().splitlines()
+    expected = playlists.SIMPLE_PLAYLIST.strip().splitlines()
     expected.remove('#EXT-X-ENDLIST')
 
     assert expected == obj.dumps().strip().splitlines()
 
+
 def test_should_dump_multiple_keys():
     obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS)
-    expected  = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS.strip()
+    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_WITH_MULTIPLE_KEYS_SORTED.strip()
 
     assert expected == obj.dumps().strip()
 
+
+def test_should_dump_unencrypted_encrypted_keys_together():
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_MULTIPLE_KEYS_UNENCRYPTED_AND_ENCRYPTED)
+    expected = playlists.PLAYLIST_WITH_MULTIPLE_KEYS_UNENCRYPTED_AND_ENCRYPTED.strip()
+
+    assert expected == obj.dumps().strip()
+
+
+def test_should_dump_complex_unencrypted_encrypted_keys():
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_MULTIPLE_KEYS_UNENCRYPTED_AND_ENCRYPTED_NONE)
+    expected = playlists.PLAYLIST_WITH_MULTIPLE_KEYS_UNENCRYPTED_AND_ENCRYPTED_NONE \
+        .replace('METHOD=NONE,URI=""', 'METHOD=NONE') \
+        .strip()
+
+    assert expected == obj.dumps().strip()
+
+
+def test_length_segments_by_key():
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_MULTIPLE_KEYS_UNENCRYPTED_AND_ENCRYPTED)
+
+    assert len(obj.segments.by_key(obj.keys[0])) == 2
+    assert len(obj.segments.by_key(obj.keys[1])) == 4
+    assert len(obj.segments.by_key(obj.keys[2])) == 2
+
+
+def test_list_segments_by_key():
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_MULTIPLE_KEYS_UNENCRYPTED_AND_ENCRYPTED)
+
+    # unencrypted segments
+    segments = obj.segments.by_key(None)
+    expected = "../../../../hls/streamNum82400.ts\n../../../../hls/streamNum82401.ts"
+    output = [ segment.uri for segment in segments ]
+    assert "\n".join(output).strip() == expected.strip()
+
+    # segments for last key
+    segments = obj.segments.by_key(obj.keys[2])
+    expected = "../../../../hls/streamNum82404.ts\n../../../../hls/streamNum82405.ts"
+    output = [ segment.uri for segment in segments ]
+    assert "\n".join(output).strip() == expected.strip()
+
+
+def test_replace_segment_key():
+    obj = m3u8.M3U8(playlists.PLAYLIST_WITH_MULTIPLE_KEYS_UNENCRYPTED_AND_ENCRYPTED)
+
+    # Replace unencrypted segments with new key
+    new_key = Key("AES-128", "/hls-key/key0.bin", None, iv="0Xcafe8f758ca555115584bb5b3c687f52")
+    for segment in obj.segments.by_key(None):
+        segment.key = new_key
+
+    # Check dump
+    expected = playlists.PLAYLIST_WITH_MULTIPLE_KEYS_UNENCRYPTED_AND_ENCRYPTED_UPDATED.strip()
+
+    assert obj.dumps().strip() == expected
+
+
 def test_should_dump_program_datetime_and_discontinuity():
     obj = m3u8.M3U8(playlists.DISCONTINUITY_PLAYLIST_WITH_PROGRAM_DATE_TIME)
-    expected  = playlists.DISCONTINUITY_PLAYLIST_WITH_PROGRAM_DATE_TIME.strip()
+    expected = playlists.DISCONTINUITY_PLAYLIST_WITH_PROGRAM_DATE_TIME.strip()
 
     assert expected == obj.dumps().strip()
 
+
 def test_should_normalize_segments_and_key_urls_if_base_path_passed_to_constructor():
     base_path = 'http://videoserver.com/hls/live'
 
     obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV, base_path)
 
-    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV \
+    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_SORTED \
         .replace(', IV', ',IV') \
         .replace('../../../../hls', base_path) \
         .replace('/hls-key', base_path) \
@@ -458,6 +566,7 @@ def test_should_normalize_segments_and_key_urls_if_base_path_passed_to_construct
 
     assert obj.dumps().strip() == expected
 
+
 def test_should_normalize_variant_streams_urls_if_base_path_passed_to_constructor():
     base_path = 'http://videoserver.com/hls/live'
     obj = m3u8.M3U8(playlists.VARIANT_PLAYLIST, base_path)
@@ -469,13 +578,14 @@ def test_should_normalize_variant_streams_urls_if_base_path_passed_to_constructo
 
     assert obj.dumps().strip() == expected
 
+
 def test_should_normalize_segments_and_key_urls_if_base_path_attribute_updated():
     base_path = 'http://videoserver.com/hls/live'
 
     obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV)
     obj.base_path = base_path     # update later
 
-    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV \
+    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_SORTED \
         .replace(', IV', ',IV') \
         .replace('../../../../hls', base_path) \
         .replace('/hls-key', base_path) \
@@ -483,13 +593,14 @@ def test_should_normalize_segments_and_key_urls_if_base_path_attribute_updated()
 
     assert obj.dumps() == expected
 
+
 def test_should_normalize_segments_and_key_urls_if_base_path_attribute_updated():
     base_path = 'http://videoserver.com/hls/live'
 
     obj = m3u8.M3U8(playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV)
     obj.base_path = base_path
 
-    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV \
+    expected = playlists.PLAYLIST_WITH_ENCRIPTED_SEGMENTS_AND_IV_SORTED \
         .replace(', IV', ',IV') \
         .replace('../../../../hls', base_path) \
         .replace('/hls-key', base_path) \
@@ -497,6 +608,7 @@ def test_should_normalize_segments_and_key_urls_if_base_path_attribute_updated()
 
     assert obj.dumps().strip() == expected
 
+
 def test_playlist_type_dumped_to_appropriate_m3u8_field():
     obj = m3u8.M3U8()
     obj.playlist_type = 'vod'
@@ -504,6 +616,7 @@ def test_playlist_type_dumped_to_appropriate_m3u8_field():
     expected = '#EXTM3U\n#EXT-X-PLAYLIST-TYPE:VOD\n'
     assert result == expected
 
+
 def test_empty_playlist_type_is_gracefully_ignored():
     obj = m3u8.M3U8()
     obj.playlist_type = ''
@@ -511,6 +624,7 @@ def test_empty_playlist_type_is_gracefully_ignored():
     expected = '#EXTM3U\n'
     assert result == expected
 
+
 def test_none_playlist_type_is_gracefully_ignored():
     obj = m3u8.M3U8()
     obj.playlist_type = None
@@ -518,6 +632,7 @@ def test_none_playlist_type_is_gracefully_ignored():
     expected = '#EXTM3U\n'
     assert result == expected
 
+
 def test_0_media_sequence_added_to_file():
     obj = m3u8.M3U8()
     obj.media_sequence = 0
@@ -525,6 +640,7 @@ def test_0_media_sequence_added_to_file():
     expected = '#EXTM3U\n'
     assert result == expected
 
+
 def test_none_media_sequence_gracefully_ignored():
     obj = m3u8.M3U8()
     obj.media_sequence = None
@@ -532,12 +648,14 @@ def test_none_media_sequence_gracefully_ignored():
     expected = '#EXTM3U\n'
     assert result == expected
 
+
 def test_should_correctly_update_base_path_if_its_blank():
     segment = Segment('entire.ts', 'http://1.2/')
     assert not segment.base_path
     segment.base_path = "base_path"
     assert "http://1.2/base_path/entire.ts" == segment.absolute_uri
 
+
 def test_m3u8_should_propagate_base_uri_to_segments():
     with open(playlists.RELATIVE_PLAYLIST_FILENAME) as f:
         content = f.read()
@@ -552,15 +670,16 @@ def test_m3u8_should_propagate_base_uri_to_segments():
     assert 'entire4.ts' == obj.segments[3].uri
     assert '/any/where/entire4.ts' == obj.segments[3].absolute_uri
 
+
 def test_m3u8_should_propagate_base_uri_to_key():
     with open(playlists.RELATIVE_PLAYLIST_FILENAME) as f:
         content = f.read()
     obj = m3u8.M3U8(content, base_uri='/any/path')
-    assert '../key.bin' == obj.key.uri
-    assert '/any/key.bin' == obj.key.absolute_uri
+    assert '../key.bin' == obj.keys[0].uri
+    assert '/any/key.bin' == obj.keys[0].absolute_uri
     obj.base_uri = '/any/where/'
-    assert '../key.bin' == obj.key.uri
-    assert '/any/key.bin' == obj.key.absolute_uri
+    assert '../key.bin' == obj.keys[0].uri
+    assert '/any/key.bin' == obj.keys[0].absolute_uri
 
 
 # custom asserts
